@@ -5,6 +5,7 @@ from sqlalchemy.orm import relationship
 from FreeTAKServer.model.SQLAlchemy.ExCheckData import ExCheckData
 from sqlalchemy import DateTime
 from sqlalchemy import Integer
+from sqlalchemy import ForeignKey
 
 
 class ExCheck(Base):
@@ -12,4 +13,5 @@ class ExCheck(Base):
     PrimaryKey = Column(Integer, primary_key = True, autoincrement=True)
     timestamp = Column(DateTime)
     creatorUid = Column(String)
-    data = relationship("ExCheckData", uselist=False)
+    checklist = relationship("ExCheckChecklist", backref='template')
+    data = relationship("ExCheckData", uselist=False, cascade="all, delete", backref="ExCheck")
